IO log fixes
[fio.git] / fio.h
diff --git a/fio.h b/fio.h
index 5ca72c59d5c707b80a326df4011b012b31e688be..b7d8582ebdcba3fc1c7b6a3c99e2b434cabbd8ed 100644 (file)
--- a/fio.h
+++ b/fio.h
 #include <string.h>
 #include <getopt.h>
 
+#include "compiler/compiler.h"
 #include "list.h"
+#include "fifo.h"
 #include "rbtree.h"
 #include "md5.h"
 #include "crc32.h"
-#include "arch.h"
-#include "os.h"
+#include "arch/arch.h"
+#include "os/os.h"
 #include "mutex.h"
 
 #ifdef FIO_HAVE_SYSLET
@@ -33,6 +35,7 @@ enum fio_ddir {
        DDIR_READ = 0,
        DDIR_WRITE,
        DDIR_SYNC,
+       DDIR_INVAL = -1,
 };
 
 enum td_ddir {
@@ -83,10 +86,17 @@ struct io_piece {
                struct rb_node rb_node;
                struct list_head list;
        };
-       struct fio_file *file;
+       union {
+               int fileno;
+               struct fio_file *file;
+       };
        unsigned long long offset;
        unsigned long len;
        enum fio_ddir ddir;
+       union {
+               unsigned long delay;
+               unsigned int file_action;
+       };
 };
 
 #ifdef FIO_HAVE_SYSLET
@@ -134,6 +144,7 @@ struct io_u {
        void *buf;
        unsigned long buflen;
        unsigned long long offset;
+       unsigned long long endpos;
 
        /*
         * IO engine state, may be different from above when we get
@@ -223,6 +234,7 @@ enum fio_filetype {
        FIO_TYPE_FILE = 1,              /* plain file */
        FIO_TYPE_BD,                    /* block device */
        FIO_TYPE_CHAR,                  /* character device */
+       FIO_TYPE_PIPE,                  /* pipe */
 };
 
 enum fio_ioengine_flags {
@@ -235,12 +247,12 @@ enum fio_ioengine_flags {
 
 enum fio_file_flags {
        FIO_FILE_OPEN           = 1 << 0,       /* file is open */
-       FIO_FILE_UNLINK         = 1 << 1,       /* unlink on close */
-       FIO_FILE_CLOSING        = 1 << 2,       /* file being closed */
-       FIO_FILE_EXISTS         = 1 << 3,       /* file there */
-       FIO_FILE_EXTEND         = 1 << 4,       /* needs extend */
-       FIO_FILE_NOSORT         = 1 << 5,       /* don't sort verify blocks */
-       FIO_FILE_DONE           = 1 << 6,       /* io completed to this file */
+       FIO_FILE_CLOSING        = 1 << 1,       /* file being closed */
+       FIO_FILE_EXISTS         = 1 << 2,       /* file there */
+       FIO_FILE_EXTEND         = 1 << 3,       /* needs extend */
+       FIO_FILE_NOSORT         = 1 << 4,       /* don't sort verify blocks */
+       FIO_FILE_DONE           = 1 << 5,       /* io completed to this file */
+       FIO_SIZE_KNOWN          = 1 << 6,       /* size has been set */
 };
 
 /*
@@ -263,6 +275,7 @@ struct fio_file {
         */
        char *file_name;
        void *mmap;
+       unsigned int major, minor;
 
        /*
         * size of the file, offset into file, and io size from that offset
@@ -289,7 +302,8 @@ struct fio_file {
  * How many depth levels to log
  */
 #define FIO_IO_U_MAP_NR        8
-#define FIO_IO_U_LAT_NR 12
+#define FIO_IO_U_LAT_U_NR 10
+#define FIO_IO_U_LAT_M_NR 12
 
 struct thread_stat {
        char *name;
@@ -327,8 +341,10 @@ struct thread_stat {
         * IO depth and latency stats
         */
        unsigned int io_u_map[FIO_IO_U_MAP_NR];
-       unsigned int io_u_lat[FIO_IO_U_LAT_NR];
+       unsigned int io_u_lat_u[FIO_IO_U_LAT_U_NR];
+       unsigned int io_u_lat_m[FIO_IO_U_LAT_M_NR];
        unsigned long total_io_u[2];
+       unsigned long short_io_u[2];
 
        unsigned long long io_bytes[2];
        unsigned long runtime[2];
@@ -387,7 +403,7 @@ struct thread_options {
        unsigned int thinktime_blocks;
        unsigned int fsync_blocks;
        unsigned int start_delay;
-       unsigned long timeout;
+       unsigned long long timeout;
        unsigned int overwrite;
        unsigned int bw_avg_time;
        unsigned int loops;
@@ -399,14 +415,16 @@ struct thread_options {
        unsigned int new_group;
        unsigned int numjobs;
        os_cpu_mask_t cpumask;
+       unsigned int cpumask_set;
        unsigned int iolog;
-       unsigned int read_iolog;
        unsigned int rwmixcycle;
        unsigned int rwmix[2];
        unsigned int nice;
        unsigned int file_service_type;
        unsigned int group_reporting;
        unsigned int fadvise_hint;
+       unsigned int zero_buffers;
+       unsigned int time_based;
 
        char *read_iolog_file;
        char *write_iolog_file;
@@ -447,6 +465,7 @@ struct thread_data {
        struct fio_file *files;
        unsigned int files_index;
        unsigned int nr_open_files;
+       unsigned int nr_done_files;
        unsigned int nr_normal_files;
        union {
                unsigned int next_file;
@@ -503,6 +522,7 @@ struct thread_data {
        unsigned long io_issues[2];
        unsigned long long io_blocks[2];
        unsigned long long io_bytes[2];
+       unsigned long long io_skip_bytes;
        unsigned long long this_io_bytes[2];
        unsigned long long zone_bytes;
        struct fio_sem *mutex;
@@ -515,6 +535,7 @@ struct thread_data {
        struct timeval start;   /* start of this loop */
        struct timeval epoch;   /* time job was started */
        struct timeval rw_end[2];
+       struct timeval last_issue;
        unsigned int rw_end_set[2];
 
        /*
@@ -644,8 +665,9 @@ struct disk_util {
        struct list_head list;
 
        char *name;
+       char *sysfs_root;
        char path[256];
-       dev_t dev;
+       int major, minor;
 
        struct disk_util_stat dus;
        struct disk_util_stat last_dus;
@@ -656,18 +678,18 @@ struct disk_util {
 
 #define DISK_UTIL_MSEC (250)
 
-#ifndef min
-#define min(a, b)      ((a) < (b) ? (a) : (b))
-#endif
-#ifndef max
-#define max(a, b)      ((a) > (b) ? (a) : (b))
-#endif
-
 /*
  * Log exports
  */
+enum file_log_act {
+       FIO_LOG_ADD_FILE,
+       FIO_LOG_OPEN_FILE,
+       FIO_LOG_CLOSE_FILE,
+};
+
 extern int __must_check read_iolog_get(struct thread_data *, struct io_u *);
-extern void write_iolog_put(struct thread_data *, struct io_u *);
+extern void log_io_u(struct thread_data *, struct io_u *);
+extern void log_file(struct thread_data *, struct fio_file *, enum file_log_act);
 extern int __must_check init_iolog(struct thread_data *td);
 extern void log_io_piece(struct thread_data *, struct io_u *);
 extern void prune_io_piece_log(struct thread_data *);
@@ -730,12 +752,14 @@ extern int __must_check open_files(struct thread_data *);
 extern int __must_check file_invalidate_cache(struct thread_data *, struct fio_file *);
 extern int __must_check generic_open_file(struct thread_data *, struct fio_file *);
 extern void generic_close_file(struct thread_data *, struct fio_file *);
-extern void add_file(struct thread_data *, const char *);
+extern int add_file(struct thread_data *, const char *);
 extern void get_file(struct fio_file *);
 extern void put_file(struct thread_data *, struct fio_file *);
 extern int add_dir_files(struct thread_data *, const char *);
 extern int init_random_map(struct thread_data *);
 extern void dup_files(struct thread_data *, struct thread_data *);
+extern int get_fileno(struct thread_data *, const char *);
+extern void free_release_files(struct thread_data *);
 
 /*
  * ETA/status stuff
@@ -743,6 +767,21 @@ extern void dup_files(struct thread_data *, struct thread_data *);
 extern void print_thread_status(void);
 extern void print_status_init(int);
 
+/*
+ * disk util stuff
+ */
+#ifdef FIO_HAVE_DISK_UTIL
+extern void show_disk_util(void);
+extern void disk_util_timer_arm(void);
+extern void init_disk_util(struct thread_data *);
+extern void update_io_ticks(void);
+#else
+#define show_disk_util()
+#define disk_util_timer_arm()
+#define init_disk_util(td)
+#define update_io_ticks()
+#endif
+
 /*
  * Thread life cycle. Once a thread has a runstate beyond TD_INITIALIZED, it
  * will never back again. It may cycle between running/verififying/fsyncing.
@@ -803,6 +842,14 @@ extern int __must_check td_io_commit(struct thread_data *);
 extern int __must_check td_io_open_file(struct thread_data *, struct fio_file *);
 extern void td_io_close_file(struct thread_data *, struct fio_file *);
 
+/*
+ * blktrace support
+ */
+#ifdef FIO_HAVE_BLKTRACE
+extern int is_blktrace(const char *);
+extern int load_blktrace(struct thread_data *, const char *);
+#endif
+
 /*
  * If logging output to a file, stderr should go to both stderr and f_err
  */